

/* @file Uno_Swingweight_LCD2rows.pde
  || @version: 1.1.1 (Kalibrierung mit Stangen)
  || 1.1 Anzeigebug Kalibrierung
  || 1.1.1 Sensor größere Zurücksetzbereich	
  || @date: 23.02.2023
  || @author: ©Crazydoc
  || @description: Measure Swingweight of Tennisrackets via Balance and Weight
  ||
  || @License:
  || freie Verwendung für private Zwecke
  || für kommerzielle Zwecke nur nach Genehmigung durch den Autor.
  || Programming language: Arduino Sketch C/C++ (IDE 1.8.13)
  ||
  || protected under the friendly Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License
  || http://creativecommons.org/licenses/by-nc-sa/3.0/
*/



//Part für LCD
#include <Wire.h> // Wire Bibliothek 
#include <LiquidCrystal_I2C.h>// Bibliothek für LCD mit I2C
#include <Keypad.h> // Bibliothek für Folientastatur 

#include <EEPROM.h> // Bibliothek für Permanentspeicher

LiquidCrystal_I2C lcd(0x27, 16, 2);   //Hier wird festgelegt um was für einen Display es sich handelt. In diesem Fall einer mit 16 Zeichen in 2 Zeilen.

// Struktur um die Werte im EProm zu speichern
struct Prog_Daten
{
  double  CalibrationStored;
  double  OffsetStored;
};

Prog_Daten eemDaten EEMEM {2.000000, 22.0000}; // struktur im EEPROM, Arduino erzeugt *.eep Datei
Prog_Daten ramDaten; // struktur im RAM

//Kalibrierungsvariablen der Maschine
double Calibration = 1.0; // Kalibrierungsvariable Gerät
double Offset = 0.0;
double SwingweightRef1Measured = 0;
double SwingweightRef2Measured = 0;
double SwingweightRef1 = 0;
double SwingweightRef2 = 0;

//Variable zum Übernehmen der Gerätekalibrierung und speicherung in EEProm
int code = 0; //Codevariable für Speicherung der Kalibrierung in EEPROM

//Variablen für Einzelschwungmessung (Optional)
unsigned long actualTime = 0; //aktuelle Zeit für Einzelschwungmessung
unsigned long previusTime = 0; //Zeit der letzten Einzelschwungmessung
unsigned long elapsedTime = 0; //Hilfsvariable für Einzelschwungmessung
unsigned long averageTime = 0; //Durschnittliche Dauer für Einzelschwungmessung

//Variablen für Mehrfachschwungmessung
unsigned long timeSpan = 0;  // Zeitspanne für Pendelmessung
unsigned long timeStart = 0; //Startzeit für Pendelmessung
unsigned long timeEnd = 0; //Endzeit für Pendelmessung

//Eingabewerte um das Schwunggewicht zu berechnen
double Balance = 31.7;  //Balance des Tennisschlägers in cm
double Weight = 311.7; // Gewicht des Tennisschlägers in Gramm
double HangDistance = 10.0; // POsition der Achse der Aufhängung

//Variablen für Schwungwertberechnung
double Pi = 3.1415;  // PI
double G = 9.81;  // Gravitationskonstante
double Swingweight = 0.00; // Schwunggewicht
double timePeriod = 1369.00; //Periodendauer einer Schwngung in Millisekunden



//Variablen für Amplitudenkalibrierung
double K = 1.0; // Kalibrierungsfaktor
int AngelMax1 = 0; //Maximale Auslenkung bei Messbeginn
int AngelMax2 = 0; //Maximale Auslenkung bei Messende
int sensorValueCalibratedAngel = 0; // Speichervariable Maximalauslenkung

// Sensor
int sensorPin = A0;
int sensorValue = 0;
int sensorCalibration = 0;
int sensorValueCalibrated = 0;
int sensorMax = 0;

//Counter und Schrittkette
int counter = 0;
int stepcounter = 0;

// Hilfsvariablen
bool startMeasurment = false ;
bool startInitialisation = false ;
bool flank = false ;
bool hilf = false;


// Initialisierung der Folientastatur
char key;
const byte ROWS = 4; //four rows
const byte COLS = 4; //four columns
//define the two-dimensional array on the buttons of the keypads
char hexaKeys[ROWS][COLS] =
{
  {'1', '2', '3', 'A'},
  {'4', '5', '6', 'B'},
  {'7', '8', '9', 'C'},
  {'.', '0', '#', 'D'}
};
byte rowPins[ROWS] = {9, 8, 7, 6}; //connect to the row pinouts of the keypad
byte colPins[COLS] = {5, 4, 3, 2}; //connect to the column pinouts of the keypad



//initialize einer Instanz der Klasse NewKeypad
Keypad customKeypad = Keypad( makeKeymap(hexaKeys), rowPins, colPins, ROWS, COLS);

void setup() // Wird nur einmalig bei Start ausgeführt und erklärt die Funktion der verschiedenen Tasten
{

  EEPROM.get((int)&eemDaten, ramDaten);
  Calibration = ramDaten.CalibrationStored;
  Offset = ramDaten.OffsetStored;

  if (isnan(Calibration))
  {
    Calibration = 1.0;
  }

  if (isnan(Offset))
  {
    Offset = 4.3;
  }

  Serial.begin(9600);
  Serial.println ("juhu es geht");
  lcd.begin();//Im Setup wird der LCD gestartet (anders als beim einfachen LCD Modul ohne 16,2 in den Klammern denn das wurde vorher festgelegt)



  lcd.setCursor(0, 0); //Ab hier kann das I2C LCD Modul genau wie das einfache LCD Modul programmiert werden.
  lcd.print("Swingweighttool");
  lcd.setCursor(0, 1); //Ab hier kann das I2C LCD Modul genau wie das einfache LCD Modul programmiert werden.
  lcd.print("by   Crazydoc");

  delay (2000);

  lcd.clear();
  lcd.setCursor(0, 0); //Ab hier kann das I2C LCD Modul genau wie das einfache LCD Modul programmiert werden.
  lcd.print("A==>Set Weight");
  lcd.setCursor(0, 1); //Ab hier kann das I2C LCD Modul genau wie das einfache LCD Modul programmiert werden.
  lcd.print("B==>Set Balance");

  delay (1000);

  lcd.clear();
  lcd.setCursor(0, 0); //Ab hier kann das I2C LCD Modul genau wie das einfache LCD Modul programmiert werden.
  lcd.print("C==>Set Sensor");
  lcd.setCursor(0, 1); //Ab hier kann das I2C LCD Modul genau wie das einfache LCD Modul programmiert werden.
  lcd.print("D==>Start");

  delay (1000);

  lcd.clear();
  lcd.setCursor(0, 0); //Ab hier kann das I2C LCD Modul genau wie das einfache LCD Modul programmiert werden.
  lcd.print("*==>.");
  lcd.setCursor(0, 1); //Ab hier kann das I2C LCD Modul genau wie das einfache LCD Modul programmiert werden.
  lcd.print("#==>Enter");

  delay (1000);

  lcd.clear();
  lcd.setCursor(0, 0); //Ab hier kann das I2C LCD Modul genau wie das einfache LCD Modul programmiert werden.
  lcd.print("Ready to Measure");
  lcd.setCursor(0, 1); //Ab hier kann das I2C LCD Modul genau wie das einfache LCD Modul programmiert werden.
  lcd.print("A===>B===>C===>D");

}


void loop()
{
  char customKey = customKeypad.getKey(); //Key aus Folientastatur auslesen

  sensorValue = analogRead(sensorPin); // Sensorwert auf Sensorvariable schreiben
  sensorValueCalibrated = sensorCalibration - sensorValue; //Kalibrierter Sensorwert


  //Die verschiedenen Schritte der Schrittkette können über die Folientastatur ausgewählt werden
  if (customKey )
  {
    Serial.println(customKey);

    switch (customKey)
    {
      case '.' : key = '.'; break;
      case '1' : key = '1'; break;
      case '2' : key = '2'; break;
      case '3' : key = '3'; break;
      case '4' : key = '4'; break;
      case '5' : key = '5'; break;
      case '6' : key = '6'; break;
      case '7' : key = '7'; break;
      case '8' : key = '8'; break;
      case '9' : key = '9'; break;
      case '0' : key = '0'; break;
      case 'A' : {                 //Schritt für Gewichtseingabe
          stepcounter = 1;
          lcd.clear();
          counter = 0;
          break;
        }
      case 'B' : {                //Schritt für Balanceeingabe
          stepcounter = 2;
          lcd.clear();
          counter = 0;
          break;
        } // Zahl B aktivieren
      case 'C' : {                //Schritt für Sensornullung
          stepcounter = 3;
          lcd.clear();
          counter = 0;
          break;
        } // Zahl C aktivieren
      case 'D' : {                //Schritt für Schwungwertberechnung
          stepcounter = 4;
          lcd.clear();
          counter = 0;
          break;
        } // Zahl D aktivieren
      case '#' : key = '#'; break;
      default: break;
    }


  }

  //Leerlaufprozess mit Referenziermodus
  if (stepcounter == 0 )
  {
    if (customKey == '1')
    {
      SwingweightRef1Measured = (Swingweight + Offset) / Calibration;

      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Ref1: ");
      lcd.print(Swingweight);
      lcd.print("=>");

      SwingweightRef1 = SetValue();       //Funktionsaufruf eingabe von Gleitkommazahlen

      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Reference 1:");
      lcd.setCursor(0, 1);
      lcd.print(Swingweight);
      lcd.print(" = ");
      lcd.print(SwingweightRef1);
    }


    if (customKey == '2')
    {
      SwingweightRef2Measured = (Swingweight + Offset) / Calibration;

      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Ref2: ");
      lcd.print(Swingweight);
      lcd.print("=>");

      SwingweightRef2 = SetValue();       //Funktionsaufruf eingabe von Gleitkommazahlen

      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Reference 2:");
      lcd.setCursor(0, 1);
      lcd.print(Swingweight);
      lcd.print(" = ");
      lcd.print(SwingweightRef2);
    }

    if (customKey == '0')
    {

      Calibration = (SwingweightRef2 - SwingweightRef1) / ( SwingweightRef2Measured - SwingweightRef1Measured);
      Offset = SwingweightRef1Measured * Calibration - SwingweightRef1;

      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Cal: ");
      lcd.print(Calibration, 4);
      lcd.setCursor(0, 1);
      lcd.print("Off: ");
      lcd.print(Offset, 4);      //Funktionsaufruf eingabe von Gleitkommazahlen

      delay (2000);

      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Enter Code:");
      lcd.setCursor(0, 1);
      code = SetValue();

      if (code == 1234)
      {
        ramDaten.CalibrationStored = Calibration;
        ramDaten.OffsetStored = Offset;
        EEPROM.put((int)&eemDaten, ramDaten);
        lcd.clear();
        lcd.setCursor(0, 0);
        lcd.print("     SYSTEM     ");
        lcd.setCursor(0, 1);
        lcd.print("   CALIBRATED  ");

      }
      else if (code == 4321)
      {
        Calibration = 1.0;
        Offset = 0.0;
        ramDaten.CalibrationStored = Calibration;
        ramDaten.OffsetStored = Offset;
        EEPROM.put((int)&eemDaten, ramDaten);
        lcd.clear();
        lcd.setCursor(0, 0);
        lcd.print("     SYSTEM     ");
        lcd.setCursor(0, 1);
        lcd.print(" SET TO DEFAULT ");

      }
      else
      {
        lcd.clear();
        lcd.setCursor(0, 0);
        lcd.print("      CODE      ");
        lcd.setCursor(0, 1);
        lcd.print("     WRONG      ");

      }
    }

  }

  //STEP 1: Gewichtseingabe

  if (stepcounter == 1 )
  {
    if (customKey == 'A')
    {
      lcd.setCursor(0, 0);
      lcd.print("Set Weight:");
      Weight = SetValue();       //Funktionsaufruf eingabe von Gleitkommazahlen
      lcd.clear();
    }
    lcd.setCursor(0, 0);
    lcd.print("Weight:");
    lcd.setCursor(0, 1);
    lcd.print(Weight);
    lcd.print(" g");


  }


  //STEP 2: Balanceeingabe

  if (stepcounter == 2 )
  {
    if (customKey == 'B')
    {
      lcd.setCursor(0, 0);
      lcd.print("Set Balance:");
      Balance = SetValue();     //Funktionsaufruf eingabe von Gleitkommazahlen
      lcd.clear();
    }
    lcd.setCursor(0, 0);
    lcd.print("Balance:");
    lcd.setCursor(0, 1);
    lcd.print(Balance);
    lcd.print(" cm");

  }


  //STEP 3: Sensorabgleich

  if (stepcounter == 3 )
  {

    lcd.setCursor(0, 0);
    lcd.print("calibrate Sensor");
    lcd.setCursor(0, 1);
    lcd.print(sensorValue);
    lcd.print(" ==> press #");

    if (customKey == '#')
    {

      sensorCalibration = sensorValue;
      sensorValueCalibrated = sensorCalibration - sensorValue; //aktuellen Sensorwert nullen


      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print(" Calibrated");
      lcd.setCursor(0, 1);
      lcd.print(sensorValue);
      lcd.print(" ==> 0");
      delay(1000);
      lcd.clear();
    }

  }

  //STEP 4: Auslenkungscheck

  if (stepcounter == 4 )
  {
    lcd.setCursor(0, 0);
    lcd.print("Deflect the");
    lcd.setCursor(0, 1);
    lcd.print("racket");

    if (sensorValueCalibrated > 10 || sensorValueCalibrated < -10)   // Überprüfung ob Auslenkung stattgefunden hatt
    {

      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Deflection OK");
      lcd.setCursor(0, 1);
      lcd.print(sensorValueCalibrated);

      stepcounter = stepcounter + 1;          //Weiter zu Step 5 Pendelmessung
    }


  }


  //STEP 5: Pendelmessung

  if (stepcounter == 5 )
  {



    //
    if (sensorValueCalibrated > sensorValueCalibratedAngel)
    {
      sensorValueCalibratedAngel = sensorValueCalibrated;
    }
    //
    if (sensorValueCalibrated > 0 && flank)   //Damit dieser Teil nur winmal je Schwung abgearbeitet wird wurde die Variable flank verwendet
    {

      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Measurment:");
      lcd.setCursor(0, 1);
      lcd.print(counter);
      Serial.println(sensorValueCalibrated);

      if (counter == 0)
      {
        previusTime = millis(); //Wenn counter = 0 wird die aktuelle Zeit gespeichert
      }

      actualTime = millis();
      elapsedTime = actualTime - previusTime ;
      previusTime = actualTime;

      if (counter == 3)
      {
        AngelMax1 = sensorValueCalibratedAngel;
        timeStart = millis(); //Nach 5 Schwüngen wird die Zeit gespeichert
      }


      //counter = counter + 1;


      averageTime = (averageTime + elapsedTime) / 2;



      if (counter == 8)
      {
        AngelMax2 = sensorValueCalibratedAngel;
        timeEnd = millis();               // Nach 16 Schwüngen wird die aktuelle Zeit gelesen
        timeSpan = timeEnd - timeStart;   // Zeitspanne zwischen Start und Stop ermittelen

        K = ((AngelMax1 + AngelMax2) / 2 * Pi / 1020) / (sin((AngelMax1 + AngelMax2) / 2 * Pi / 1020));
        timePeriod = timeSpan / 5 / K;       // Periodendauer eines Schwunges berechnen

        Swingweight = ((Weight / 1000) * (((Balance - HangDistance) * G * 100.0) / (4 * Pi * Pi) * pow(timePeriod / 1000, 2) - pow(Balance - HangDistance, 2) + pow(Balance - 10.0, 2))) * Calibration - Offset;
        //Swingweight=(Weight/1000)*G*100*pow(timePeriod/1000,2)*(BalanceCalibrated-HangDistance)/(4*Pi*Pi*)); kann verwendet werden wenn Rotationsachse bei 10cm liegt

        lcd.setCursor(0, 0);
        lcd.print("Time for 1 Swing:");
        lcd.setCursor(0, 1);
        lcd.print(timePeriod / 1000, 4);
        lcd.print(" s");
        delay (2000);



        lcd.clear();
        lcd.setCursor(0, 0);
        lcd.print("Swingweight:");
        lcd.setCursor(0, 1);
        lcd.print(Swingweight);
        lcd.print(" kg/cm^2");

        counter = 0;
        stepcounter = 0;
        AngelMax1 = 0;
        AngelMax2 = 0;


      }
      flank = false;

      counter = counter + 1;
      sensorValueCalibratedAngel = 0;

    }


    if (sensorValueCalibrated < -5)
    { flank = true;
    }

  }



}


//Funktion zur Gleitkommazeileneingabe auf der Folientastatur

double SetValue()
{
  double num = 0;
  bool hilf = false;
  int counterhilf = 0;

  char key = customKeypad.getKey();
  lcd.setCursor(0, 1);

  while (key != '#' )
  {
    switch (key)
    {
      case NO_KEY:
        break;

      case '0': case '1': case '2': case '3': case '4':
      case '5': case '6': case '7': case '8': case '9':

        lcd.print(key);
        num = num * 10 + (key - '0');
        if (hilf)
          counterhilf = counterhilf + 1;

        break;


      case '.':
        lcd.print(key);
        hilf = true;
        break;

      case '#':
        num = 0;

        lcd.clear();
        break;

    }

    key = customKeypad.getKey();
  }

  return num / pow(10, counterhilf);
}
